package com.ym.mec.student.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alipay.service.schema.util.StringUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.entity.Student;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.entity.UserMusicStar;
import com.ym.mec.biz.dal.enums.ClientEnum;
import com.ym.mec.biz.dal.wrapper.UserMusicStarWrapper;
import com.ym.mec.biz.service.StudentService;
import com.ym.mec.biz.service.SubjectService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.biz.service.UserMusicStarService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.student:}/userMusicStar")
@Api(tags = "用户曲目作品点赞记录")
public class UserMusicStarController  extends BaseController {

    @Autowired
    private UserMusicStarService userMusicStarService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private SysUserService sysUserService;

    @ApiOperation(value = "查询分页", notes = "用户曲目作品点赞记录- 传入 UserMusicStarWrapper.UserMusicStarQuery")
    @PostMapping("/page")
    public HttpResponseResult<PageInfo<UserMusicStarWrapper.UserMusicStar>> page(@RequestBody @Validated UserMusicStarWrapper.UserMusicStarQuery query) {

        IPage<UserMusicStar> pages = userMusicStarService.selectPage(QueryInfo.getPage(query), query);

        List<UserMusicStar> records = pages.getRecords();

        List<UserMusicStarWrapper.UserMusicStar> userMusicStars = new ArrayList<>();
        if (CollectionUtils.isEmpty(records)){
            return succeed(QueryInfo.pageInfo(pages,userMusicStars));
        }

        userMusicStars = JSON.parseArray(JSON.toJSONString(records), UserMusicStarWrapper.UserMusicStar.class);

        // 学生ID集合
        List<Integer> studentIds = records.stream().map(UserMusicStar::getUserId).map(Long::intValue).collect(Collectors.toList());

        Map<Integer, Student> studentMap = studentService.getMapByIds(studentIds);
        Map<Long, SysUser> userMap = sysUserService.getMapByIds(studentIds);

        for (UserMusicStarWrapper.UserMusicStar userMusicStar : userMusicStars) {
            Student student = studentMap.get(userMusicStar.getUserId());
            if (student != null){
                userMusicStar.setSubjectId(student.getSubjectId());
            }
            SysUser sysUser = userMap.get(userMusicStar.getUserId());
            if (sysUser != null) {
                userMusicStar.setUserName(sysUser.getUsername());
                userMusicStar.setUserAvatar(sysUser.getAvatar());
            }
        }

        // 声部ID集合
        List<Integer> subjectIds = studentMap.values().stream().map(Student::getSubjectId).filter(StringUtils::isNotBlank).map(Integer::parseInt).collect(Collectors.toList());

        Map<Integer, Subject> subjectMap =
            subjectService.getMapByIds(subjectIds);

        for (UserMusicStarWrapper.UserMusicStar userMusicStar : userMusicStars) {


            if (StringUtil.isEmpty(userMusicStar.getSubjectId())) {
                continue;
            }
            Subject subject = subjectMap.get(Integer.parseInt(userMusicStar.getSubjectId()));
            if (subject != null){
                userMusicStar.setSubjectName(subject.getName());
            }
        }

        return succeed(QueryInfo.pageInfo(pages,userMusicStars));
    }


    @ApiOperation(value = "点赞/取消点赞")
    @PostMapping("/star")
    public HttpResponseResult<JSONObject> star(@Validated @RequestBody UserMusicStarWrapper.Star star) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        star.setUserId(user.getId().longValue());
        star.setClientType(ClientEnum.STUDENT);
        // 新增数据
        userMusicStarService.star(star);

        return succeed();
    }
}
